#!/bin/bash
#

# build_tauola.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e19|e20|c7>:p382 debug|prof [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
	    case $qual in
        e19|e20|c7)
        	cqual=$qual
        	;;
        p382)
        	pqual=$qual
        	;;
        *)
        	ssi_die "Unrecognized qualifier $qual"
	    esac
    done
    if [[ -z "$cqual" ]]; then
	    ssi_die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]]; then
	    ssi_die "A python qualifier is mandatory (e.g. p382)"
    fi
    basequal=${cqual}:${pqual}
}

set_compiler_flags()
{
  cxxflg=()
  cflg=()
  fcflg=()
  case ${cqual} in
    e19|e20)
      cc=gcc
      cxx=g++
      cxxstd=17
      fc=gfortran
      ;;
    c7)
      cc=clang
      cxx=clang++
      cxxstd=17
      fc=gfortran
      ;;
    *)
      ssi_die "unrecognized compiler qualifier ${cqual}\n$(usage)"
  esac
  cxxflg+=(-std=c++${cxxstd} -Wno-deprecated-declarations -Wno-unused-variable -Wno-unused-but-set-variable)
  cflg+=(-std=c11)
  fcflg+=(-std=legacy)

  case ${extraqual} in
    debug)
      optflg+=(-g -O0)
      ;;
    prof)
      optflg+=(-g -O3 -DNDEBUG -fno-omit-frame-pointer)
      ;;
    opt)
      optflg+=(-O3 -DNDEBUG)
      ;;
    *)
      ssi_die "please specify debug, opt, or prof\n$(usage)"
      ;;
  esac
  cflg+=("${optflg[@]}")
  fcflg+=("${optflg[@]}")
  cxxflg+=("${optflg[@]}")
  # Needed for a successful build of the legacy Fortran when --prefix is
  # specified.
  cppflg+=("-I${blddir}/${pkgtardir}/include")
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z "${product_dir}" ]; then
  echo "ERROR: please specify the local product directory" 1>&2
  usage
  exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=tauola
origpkgver=v1_1_8
pkgver=${origpkgver}
ssibuildshims_version=v2_00_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=TAUOLA.${pkgdotver}
pkgtardir=${pkgtarname%%.*}
pkgtarfile=${pkgtarname}.tar.gz

get_this_dir

get_ssibuildshims

# Need in-source build for now (blddir == sourcedir).
source define_basics IN_SOURCE

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

mkdir -p ${blddir} || ssi_die "failed to create build directory ${blddir}"
mkdir -p ${sourcedir} || ssi_die "failed to create source directory ${sourcedir}"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || \
  ssi_die "unable to set up ${package} for build"

echo
ups active
echo

#ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`
ncore=1 # Bad Makefiles, Bad!


####################################
# Patches.
#
# 1. Patch configure.in to not confuse UPS XXX_VERSION variables for LCG
#    ones, and not to forcibly override CXXFLAGS.
#
# 2. Remove overriding -std=c++11 flag from make.inc.in and allow
#    user-supplied CXXFLAGS and CPPFLAGS to override.
####################################

cd "${sourcedir}" || ssi_die "unable to cd to source directory ${sourcedir}"

prepare_source -a "${pkgtardir}" -P "${patchdir}" \
  "${tardir}/${pkgtarfile}" \
  fix-configure-in.patch fix-make-inc-in.patch || exit $?

# Current TAUOLA build system requires configure be invoked from the
# source (==build) directory.
cd "${blddir}/${pkgtardir}"

set_qualifiers
set_compiler_flags

set -x

# We patched configure.in, so we should re-run autoconf.
autoconf

"${sourcedir}/${pkgtardir}/configure" --prefix="${pkgdir}" \
  --with-hepmc3="${HEPMC3_FQ_DIR}" \
  --without-hepmc \
  --with-lhapdf="${LHAPDF_FQ_DIR}" \
  --with-tau-spinner \
  CC=${cc} CFLAGS="${cppflg[*]} ${cflg[*]}" \
  CXX=${cxx} CXXFLAGS="${cppflg[*]} ${cxxflg[*]}" \
  F77=${fc} FFLAGS="${fcflg[*]}" \
  || ssi_die "configure failed"

# Now actually build.
make -j ${ncore} || ssi_die "make failed"

# Do the full install.
make -j ${ncore} install || ssi_die "make install failed"

# Tidy source to install as debug source
make clean; find . -name '*~' | xargs rm -f
cd "${pkgups}/.." && mv "${blddir}" source || \
  ssi_die "unable to move source for packaging"

set +x

[[ -d "${pkgdir}/lib" ]] || \
  ssi_die "failed to create ${pkgdir}/lib\n" \
   "      Something is very wrong"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "unable to declare installed product"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || \
  ssi_die "unable to setup installed product"
echo "tauola is installed at ${TAUOLA_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
